indexing
	description: "States, in which transition and output functions are represented by arrays that have a row for each combination of predicate values"
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	FULL_TABLE_STATE

inherit
	STATE
	redefine
		set_input
	end
create
	make, make_compatible

feature -- Initialization
	make (p, a, s: INTEGER) is
			-- Create state with `p' predicates, `a' actions and capable of connecting to `s' other states
		require
			predicate_count_not_negative: p >= 0
			action_count_not_negative: a >= 0
			state_count_positive: s > 0
		local
			rows_count, i: INTEGER
			o: BIT_VECTOR
		do
			predicate_count := p
			action_count := a
			state_count := s
			rows_count := 1 |<< p
			create output_function.make(1, rows_count)
			create transition_function.make(1, rows_count)
			create o.make (1, action_count)
			from
				i := 1
			until
				i > rows_count
			loop
				output_function[i] := o.twin
				transition_function[i] := Initial_state
				i := i + 1
			end
			create current_input.make (1, predicate_count)
			update_input_index
		ensure
			predicate_count_set: predicate_count = p
			action_count_set: action_count = a
			state_count_set: state_count = s
		end

	make_compatible (co: CONTROL_OBJECT; s: INTEGER) is
			-- Create a state with same `predicate_count' and `action_count' as `co' and capable of connecting to `s' other states
		require
			co_exists: co /= Void
		do
			make(co.predicate_count, co.action_count, s)
		ensure
			predicate_count_compatible: predicate_count = co.predicate_count
			action_count_compatible: action_count = co.action_count
			state_count_set: state_count = s
		end

feature -- Access
	predicate_count: INTEGER
			-- Number of predicates in input to output function and transition function

	action_count: INTEGER
			-- Number of actions in output function's values

	state_count: INTEGER
			-- Potential capacity of transition function value space

feature {AUTO} -- Transformation
	set_input (in: BIT_VECTOR) is
		-- Set `current_input' to `in'
		do
			current_input.copy (in)
			update_input_index
		end

feature {AUTO} -- Access
	actions: BIT_VECTOR is
			-- Value of output function for `current_input'
		do
			Result := output_function[input_index].twin
		end

	next_state: INTEGER is
			-- Value of transition function for `current_input'
		do
			Result := transition_function[input_index]
		end

feature {FULL_TABLE_STATE} -- Transformation
	set_output_function (of: ARRAY[BIT_VECTOR]) is
			-- Set `output_function' to `of'
		require
			of_exists: of /= Void
			of_rows_count_correct: of.count = 2 ^ predicate_count
			of_lower_correct: of.lower = transition_function.lower
		do
			output_function := of
		ensure
			output_function_set: output_function = of
		end

	set_transition_function (tf: ARRAY[INTEGER]) is
			-- Set `transition_function' to `tf'
		require
			tf_exists: tf /= Void
			tf_rows_count_correct: tf.count = 2 ^ predicate_count
			tf_lower_correct: tf.lower = output_function.lower
		do
			transition_function := tf
		ensure
			transition_function_set: transition_function = tf
		end

feature --{FULL_TABLE_STATE} -- Implementation
	current_input: BIT_VECTOR
			-- Input to be used by `actions' and `next_state'

	output_function: ARRAY[BIT_VECTOR]
			-- Output function representation

	transition_function: ARRAY[INTEGER]
			-- Transition function representation

	input_index: INTEGER
			-- Integer representation of `current_input'

	update_input_index is
			-- Make `input_index' a correct representation of `current_input'
		do
			input_index := current_input.to_integer + 1
		end

invariant
	output_function_exists: output_function /= Void
	transition_function_exists: transition_function /= Void
	transition_function_rows_count_correct: transition_function.count = 2 ^ predicate_count
	output_function_rows_count_correct: output_function.count = 2 ^ predicate_count
	lower_the_same: output_function.lower = transition_function.lower
	upper_the_same: output_function.upper = transition_function.upper
	input_index_not_too_small: input_index >= 1
	input_index_not_too_large: input_index <= 2 ^ predicate_count
end
