class POLLING_SERVER

inherit STORABLE

create
	make

feature
	knot: REAL is 0.5144444

	make (argv: ARRAY [STRING]) is
		do
			generate_auto
		end

	brk: X_PLANE_BROKER

feature -- Predicates

	no_stall_with_flaps: BOOLEAN is
			-- Whether plane speed is enough not to stall, having flaps down
		do
			Result := brk.airspeed_knots > 60
		end
	no_stall_without_flaps: BOOLEAN is
			-- Whether plane speed is enough not to stall, having flaps up
		do
			Result := brk.airspeed_knots > 70
		end
	flaps_allowed: BOOLEAN is
			-- Whether plane speed is small enough to permit flaps
		do
			Result := brk.airspeed_knots < 120
		end
	high_speed: BOOLEAN is
			-- Whether plane flyes faster its cruise speed
		do
			Result := brk.airspeed_knots > 172
		end
	critically_high_speed: BOOLEAN is
			-- Whether plane flyes faster its maximum allowed speed
		do
			Result := brk.airspeed_knots > 200
		end

	positive_acceleration: BOOLEAN is
			-- Whither plane is accelerating
		do
			Result := brk.acceleration > 0
		end
	low_acceleration: BOOLEAN is
			-- Whether acceleration module is low
		do
			Result := brk.acceleration.abs < 1
		end
	high_acceleration: BOOLEAN is
			-- Whether acceleration module is high
		do
			Result := brk.acceleration.abs > 4
		end

	positive_roll: BOOLEAN is
			-- Whether roll is positive
		do
			Result := brk.roll > 0
		end
	positive_roll_speed: BOOLEAN is
			-- Whether roll speed is positive
		do
			Result := brk.d_roll > 0
		end
	low_roll_speed: BOOLEAN is
			-- Whether roll speed is low
		do
			Result := brk.d_roll.abs < 1
		end
	high_roll_speed: BOOLEAN is
			-- Whether roll speed is high
		do
			Result := brk.d_roll.abs > 4
		end

	positive_ils1_hdef: BOOLEAN is
			-- Whether horizontal deflection from ILS is positive
		do
			Result := brk.vor1_hdef > 0
		end
	positive_ils1_hdef_speed: BOOLEAN is
			-- Whether horizontal deflection from ILS derivative is positive
		do
			Result := brk.d_vor1_hdef > 0
		end
	low_ils1_hdef_speed: BOOLEAN is
			-- Whether horizontal deflection from ILS derivative is low
		do
			Result := brk.d_vor1_hdef.abs < 0.1
		end
	high_ils1_hdef_speed: BOOLEAN is
			-- Whether horizontal deflection from ILS derivative is high
		do
			Result := brk.d_vor1_hdef.abs > 0.4
		end



feature -- AutoGen
	generate_auto is
		local
			gs: GENETIC_REDUCED_TABLE_STATE
			ga: GENETIC_AUTO
			gao: GENETIC_AUTOMATED_OBJECT
			sf: XPLANE_CO
			e: EVOLUTION
			fn: STRING
			i: INTEGER
			f: PLAIN_TEXT_FILE
		do
			create sf.make
			create gs.make_compatible (sf, 12, 1)
			gs.set_mutation_index (0.001)
			create ga.make(gs)
			create gao.make(ga, sf)
			create e.make(gao)

			create fitnesses.make

			e.set_population_size (100)
			e.set_survivers_part (0.1)
			print ("Current population best individual's fitness")
			io.new_line
			from
				create f.make (file_name2)
				if f.exists then
					e ?= retrieve_by_name (file_name2)
					if e = Void then
						create e.make(gao)
						e.set_population_size (100)
						e.set_survivers_part (0.1)
						e.start
					end
				else
					e.start
				end
				i := 0
			until
				sf.timeout_sec = 300
			loop
				gao ?= e.best
				print (gao.fitness)
				io.put_string (" (")
				io.put_integer (sf.timeout_sec)
				io.put_string (" s)")
				io.new_line
				sf.update_timeout (gao.fitness)
				fitnesses.put_front (gao.fitness)
				if fitnesses.count > n then
					fitnesses.finish
					fitnesses.remove
				end

				e.forth
				i := i + 1
				create fn.make_from_string (file_name2)
				fn.append_integer (i)
				e.store_by_name (fn)
				e.store_by_name (file_name2)
			end
			print ("Optimization finished")
			io.new_line
			print ("Resulting automaton stored in file: ")
			print (file_name)
			io.read_character
	end

	save_auto is
		local
			gs: GENETIC_REDUCED_TABLE_STATE
			ga: GENETIC_AUTO
			gao: GENETIC_AUTOMATED_OBJECT
			sf: XPLANE_CO
			e: EVOLUTION
			f: PLAIN_TEXT_FILE
			inds: ARRAY [INDIVIDUAL]
			s: STRING
			i: INTEGER
		do
			create sf.make
			create gs.make_compatible (sf, 12, 1)
			create ga.make(gs)
			create gao.make(ga, sf)
			create e.make(gao)

			create fitnesses.make

			e.set_population_size (20)
			e.set_survivers_part (0.1001)
			create f.make (file_name2)
			e ?= retrieve_by_name (file_name2)
			inds := e.inds
			from i := inds.lower
			until i > inds.upper
			loop
				gao ?= inds.item (i)
				ga ?= gao.get_controller
				create s.make_from_string ("auto")
				s.append_integer (i)
				s.append (".txt")
				ga.to_file (s)
				io.put_integer (i)
				io.put_string (": ")
				io.put_real (gao.fitness)
				io.put_new_line
				i := i + 1
			end
	end

	test_auto is
		local
			i: INTEGER
			gao: GENETIC_AUTOMATED_OBJECT
		do
			gao ?= retrieve_by_name (file_name)
			gao.restart
			from
				i := 1
			until
				False
			loop
				gao.show_computational_state
				gao.do_step
				i := i + 1
				io.read_character
			end
		end

feature {NONE} -- Implementation of AutoGen
	file_name: STRING is
			-- File where optimization result is stored
		once
			Result := "result"
		end
	file_name2: STRING is
			-- File where optimization result is stored
		once
			Result := "population"
		end

	n: INTEGER is 10
			-- How many generations are taken to calculate average fitness

	fitnesses: LINKED_LIST[REAL]
			-- Fitness values of last `n' generation's best individuals

	avg_fitness: REAL is
			-- Average value of `fitnesses'
		require
			fitnesses_exist: fitnesses /= Void
		do
			from fitnesses.start
			until fitnesses.after
			loop
				Result := Result + fitnesses.item
				fitnesses.forth
			end
			Result := Result / n
		end

end -- POLLING_SERVER

